//日期格式化
Date.prototype.format = function(fmt) {
    const o = {
        "M+" : this.getMonth()+1,                 //月份
        "d+" : this.getDate(),                    //日
        "h+" : this.getHours(),                   //小时
        "m+" : this.getMinutes(),                 //分
        "s+" : this.getSeconds(),                 //秒
        "q+" : Math.floor((this.getMonth()+3)/3), //季度
        "S"  : this.getMilliseconds()             //毫秒
    };
    if(/(y+)/.test(fmt)) {
        fmt=fmt.replace(RegExp.$1, (this.getFullYear()+"").substr(4 - RegExp.$1.length));
    }
    for(const k in o) {
        if(new RegExp("("+ k +")").test(fmt)){
            fmt = fmt.replace(RegExp.$1, (RegExp.$1.length==1) ? (o[k]) : (("00"+ o[k]).substr((""+ o[k]).length)));
        }
    }
    return fmt;
}

//获取农历的月日
Date.prototype.getChinese = function(){
    const arr = this.toLocaleString('ja-JP-u-ca-chinese').split('-');
    let month = parseInt(arr[1]);
    let day = parseInt(arr[2]);
    if(month < 10){
        month = '0' + month;
    }
    if(day < 10){
        day = '0' + day;
    }
    return month + '' + day;
}

//获取2个时间之间相差多少天
function getPassDay(day1,day2){
    const season = getSeason();
    const rate = season === '春' ? 2 : season === '冬' ? 0 : 1;
    return parseInt((day2 - day1) * rate / (1000 * 60 * 60 * 24))
}

//获取2个时间之间相差多少分钟
function getPassMin(day1,day2){
    return parseInt((day2 - day1) / (1000 * 60))
}

//给指定时间增加指定天数
function addDay(date,num){
    return new Date(date.getTime() + (num * 1000 * 60 * 60 * 24))
}

//给指定时间增加指定分钟
function addMin(date,num){
    return new Date(date.getTime() + (num * 1000 * 60))
}

//打乱数组
if (!Array.prototype.derangedArray) {
    Array.prototype.derangedArray = function() {
        for(let j, x, i = this.length; i; j = parseInt(Math.random() * i), x = this[--i], this[i] = this[j], this[j] = x);
        return this;
    };
}

//生成从minNum到maxNum的随机数
function randomNum(minNum = 1,maxNum = 100){
    return parseInt(Math.random()*(maxNum-minNum+1)+minNum,10)
}

//获取数组中的随机一个值
Array.prototype.getRandom = function(num = 1){
    if(num === 1){
        return this[randomNum(0, this.length - 1)];
    }
    if(num > this.length){
        num = this.length;
    }
    const arr = [];
    const indexs = [];
    for(let i = 0 ; i < num ; i++){
        let index = randomNum(0, this.length - 1);
        while (indexs.indexOf(index) !== -1){
            index = randomNum(0, this.length - 1);
        }
        indexs.push(index);
        arr.push(this[index])
    }
    return arr;
}

//是否包含指定集合里所有元素
Array.prototype.hasAll = function(arr = []){
    return !(arr.filter(e => this.indexOf(e) === -1).length > 0)
}

//是否和指定数组有交集
Array.prototype.hasOne = function(arr = []){
    for(let i = 0 ; i < this.length ; i++){
        if(arr.includes(this[i])){
            return true;
        }
    }
    return false;
}

//保留两位小数的解析浮点型
function parseFloat2Fiexd(val){
    return parseFloat(parseFloat(val).toFixed(2));
}

//数组去重
function arrayDeweight(arr){
    return [...new Set(arr)]
}

//多个数组取交集
function arrayIntersection(arr){
    return arr.reduce((a, b) => a.filter(c => b.includes(c)));
}

/**
 * 计算百分比
 * @param   {number} num   分子
 * @param   {number} total 分母
 * @returns {number} 返回数百分比
 */
function Percentage(num, total) {
    if (!num || !total){
        return 0;
    }
    return (Math.round(num / total * 10000) / 100.00);// 小数点后两位百分比
}

//创建4位ID
function S4() {
    return (((1+Math.random())*0x10000)|0).toString(16).substring(1);
}

//创建32位ID
function getNewId() {
    return ""+S4()+S4()+S4()+S4()+S4()+S4()+S4()+S4();
}

//localStorage封装工具,自动进行json格式转换
const Store = {
    get : function(key){
        let value = localStorage.getItem(key);
       /* $.ajax({
            url: "temp/" + key + ".saved",
            async: false,
            success(resp){
                value = resp
            }
        });*/
        try {
            value = JSON.parse(value);
        }catch (e) {
            console.error(e)}
        return value;
    },
    set : function(key,value){
        let val = null;
        try {
            val = JSON.stringify(value);
            localStorage.setItem(key,val || value);
        }catch (e) {
            console.error(e)}
    },
    remove : function(key = (diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName){
        save(`${conf.savedName}-backup`);
        saveWorlds(conf.savedName + '-map-backup')
        delAllSavedFile(conf.savedName);
    }
};

//删除所有存档相关文件
function delAllSavedFile(savedName){
    const name = (diff === '正常难度' || diff == '0' ? '' : diff) + savedName;
    if(window.control){
        window.control.del(name + `-backup`);
        window.control.del(name + '-map-backup');
        window.control.del(name);
        window.control.del(name + '-map');
    }else{
        localStorage.removeItem(name + `-backup`);
        localStorage.removeItem(name + '-map-backup');
        localStorage.removeItem(name);
        localStorage.removeItem(name + '-map');
    }
}

//存档
let needSave = false;
let isDie = false;
let canSaveExit = true;

function save(name = conf.savedName, saveData = data){
    canSaveExit = false;
    try {
        //成就判定
        checkAchievement();
    }catch (e) {
        console.error(e)}

    //每次要存档时，记录当前世界BOSS的属性
    try {
        let finalBoss = null;
        const map = data.maps.filter(e => e.name === data.world.name);
        if(map.length > 0){
            finalBoss = map[0].finalBoss;
        }
        if(data.battleMonster && data.battleMonster.name === finalBoss){
            data.world.finalBossObj = data.battleMonster;
        }
    }catch (e) {
        console.error(e)}

    if(name !== conf.savedName){
        let val = null;
        try {
            val = JSON.stringify(saveData);
        }catch (e) {
            console.error(e)}
        try {
            window.control.save(`${(diff === '正常难度' || diff == '0' ? '' : diff) + name}###---###${val || saveData}`);
        }catch (e) {
            if(window.control){
                msg('存档失败：' + e)
            }
            Store.set((diff === '正常难度' || diff == '0' ? '' : diff) + name,saveData);
        }
    }else{
        needSave = true;
    }
}

//备份存档
function backupSaved(keep = true){
    const {worlds, ...datas} = data;
    save(`${conf.savedName}-backup`, datas);
    saveWorlds(conf.savedName + '-map-backup')

    if(keep){
        setTimeout(backupSaved, randomNum(5,10) * 60 * 1000)
    }
}

setTimeout(backupSaved, randomNum(5,10) * 60 * 1000)

//还原存档
function restoreSaved(){
    if(data.order < 2 && data.worlds.length < 3 && data.level < 30){
        alert('至少30级，或1周目以上，或打到了半岛，才能使用存档还原功能');
        return;
    }
    let restoreNum = data.restoreNum;
    if(restoreNum >= 3){
        alert('你已经还原过3次存档了，该存档已无法再还原');
        return;
    }
    if(!confirm(`每个存档最多只能还原3次，你已经还原了${restoreNum || 0}次。还原之前请一定要看一下图鉴-说明-存档还原，否则指不定会有啥后果！确定要还原吗？`)){
        return;
    }
    needSave = false;
    const loadData = load(`${conf.savedName}-backup`);
    if(loadData){
        data = loadData

        //增加存档还原次数
        data.restoreNum = restoreNum + 1;

        if(window.control){
            const resp = window.control.load((diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName + "-map-backup");
            data.worlds = resp ? JSON.parse(resp) : [];
        }else{
            data.worlds = Store.get((diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName + "-map-backup") || [];
        }

        let val = null;
        try {
            val = JSON.stringify(data);
        }catch (e) {
            console.error(e)}
        try {
            window.control.save(`${(diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName}###---###${val || data}`);
        }catch (e) {
            if(window.control){
                msg('存档失败：' + e)
            }
            Store.set((diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName,data);
        }
        saveWorlds();
        $('.alert-win').show().find('.alert-win-title').html('请勿操作游戏，2秒后还原成功')
        setTimeout(() => {
            window.location.reload();
        }, 2000)
    }else{
        winMsg('不存在已备份的存档');
    }
}

//自动存档
setInterval(function(){
    if(needSave && !isDie){
        needSave = false;

        const {worlds, ...datas} = data;
        if(window.control){
            let val = null;
            try {
                val = JSON.stringify(datas);
            }catch (e) {
                console.error(e)}
            window.control.save(`${(diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName}###---###${val || datas}`);
            canSaveExit = true;
        }else{
            Store.set((diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName,datas);
            msg('自动存档成功', false);
        }
        systemLock = false;
    }
},2000)

//安全退出
$('.safe-exit').click(function(){
    if(!window.control){
        return;
    }
    $('.alert-win').show().find('.alert-win-title').html('系统正在安全退出，请稍等……')
    setInterval(function(){
        if(canSaveExit){
            window.control.exit();
        }
    },500);
});

//加载
function load(name = conf.savedName){
    if(window.control){
        const resp = window.control.load((diff === '正常难度' || diff == '0' ? '' : diff) + name);
        return resp ? JSON.parse(resp) : null;
    }else{
        return Store.get((diff === '正常难度' || diff == '0' ? '' : diff) + name);
    }
}

//加载配置
function loadConf(){
    if(window.control){
        const resp = window.control.load((diff === '正常难度' || diff == '0' ? '' : diff) + confSavedName);
        return new Conf(resp ? JSON.parse(resp) : {});
    }
    if(Store.get((diff === '正常难度' || diff == '0' ? '' : diff) + confSavedName)){
        return new Conf(Store.get((diff === '正常难度' || diff == '0' ? '' : diff) + confSavedName));
    }

    return new Conf({});
}

//加载世界
function loadWorlds(){
    if(window.control){
        const resp = window.control.load((diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName + "-map");
        data.worlds = resp ? JSON.parse(resp) : [];
    }else{
        data.worlds = Store.get((diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName + "-map") || [];
    }
}

//保存世界
function saveWorlds(name = conf.savedName + '-map'){
    try {
        //成就判定
        save(name, data.worlds)
    }catch (e) {
        console.error(e)}
}

//加载难度
function loadDiff(){
    return window.control ? window.control.load("diff") : Store.get("diff");
}

//加载难度
function saveDiff(){
    window.control ?  window.control.save(`diff###---###${diff || '0'}`) : Store.set("diff", diff || '0');
}

//批量创建物品
function getBatchItem(arr = [], obj){
    let data = {};
    arr.forEach(e => {
        if(typeof obj === 'function'){
            data = {
                ...data,
                ...obj(e)
            }
        }else{
            data[e] = obj;
        }
    })
    return data;
}

//以下为作者调试用的方法

//获取指定地图可以采集到的所有物品
function getAllElesByMapName(name){
    let arr = [];
    terrainArray[name].forEach(e => {
        Object.keys(terrainElements[e].elements).forEach(el => {
            arr = [...arr, ...elementCfgs[el].getItem.map(i => i.name)]
        })
    })

    return [...new Set(arr)];
}
//获取指定地图可以打怪掉落的所有物品
function getAllMonsterItemByMapName(name){
    const level = allMapList.filter(m => m.name === name)[0].level;
    let arr = [];
    terrainArray[name].forEach(e => {
        Object.keys(terrainElements[e].monster).filter(el => monsterCfgs[el].level >= level[0] && monsterCfgs[el].level <= level[1]).forEach(el => {
            arr = [...arr, ...monsterCfgs[el].items.map(i => i.name)]
        })
    })

    return [...new Set(arr.filter(e => !e.startsWith('图纸')))];
}

//点亮全部地图
function showAllMap(){
    for(let y = 0 ; y < ynum ; y++){
        for(let x = 0 ; x < xnum ; x++){
            data.world.showMap[`${x},${y}`] = 1;
        }
    }
}

//点击查看系统
$('.show-system').click(function(){
    try{
        $('#world-name').html(data.world.name);
        $('#world-min-level').html(data.world.level[0]);
        $('#world-max-level').html(data.world.level[1]);
        $('.myth-msg').html(`神话怪提醒：${conf.mythMsg ? '已开启' : '已关闭'}`);
        $('.legend-msg').html(`传说怪提醒：${conf.legendMsg ? '已开启' : '已关闭'}`);
        $('.elite-msg').html(`精英怪提醒：${conf.eliteMsg ? '已开启' : '已关闭'}`);
        $('.ins-msg').html(`灵感提醒：${conf.insMsg ? '已开启' : '已关闭'}`);
        $('.hp-warning').html(`血量提醒：${conf.hpWarning ? '已开启' : '已关闭'}`);
        $('.hunger-warning').html(`饥饿提醒：${conf.hungerWarning ? '已开启' : '已关闭'}`);
        $('.water-warning').html(`水分提醒：${conf.waterWarning ? '已开启' : '已关闭'}`);
        $('.weight-warning').html(`重量提醒：${conf.weightWarning ? '已开启' : '已关闭'}`);
        $('.show-auto-battle').html(`自动攻击：${conf.showAutoBattle ? '已开启' : '已关闭'}`);
        $('.keep-atk').html(`连续攻击：${conf.keepAtk ? '已开启' : '已关闭'}`);
        $('.dodge-btn').html(`闪避按钮：${conf.showDodgeBtn ? '已开启' : '已关闭'}`);
        $('.is-bottom-win').html(`关闭按钮位置：${conf.isBottomWin ? '下方' : '上方'}`);
        $('.home-num').html(`家设施列数：${conf.homeNum || 1}`);
        $('.home-order').html(`家设施排序：${conf.homeOrder ? '已开启' : '已关闭'}`);
        $('.monster-sort').html(`怪物排序：${conf.monsterSort ? '已开启' : '已关闭'}`);
        $('.dark-model').html(`夜间模式：${conf.darkModel ? '已开启' : '已关闭'}`);
        $('.monster-level').html(`怪物等级：${conf.showMonsterLevel ? '显示' : '隐藏'}`);
        $('.season-day-night-show').html(`季节日夜：${conf.showSeasonDayNight ? '显示' : '隐藏'}`);
        $('.myth-status-show').html(`神话存活提醒：${conf.showMythStatus ? '显示' : '隐藏'}`);save();
        $('.ele-filter-show').html(`页面元素过滤：${conf.showEleFilter ? '显示' : '隐藏'}`);
        $('.show-verification').html(`检定记录：${conf.showVerification ? '已开启' : '已关闭'}`);
        $('.close-chat-room').html(`聊天室：${conf.closeChatRoom ? '已关闭' : '已开启'}`);
        $('.hide-map-btn-box').html(`移动键盘：${conf.hideMapBtnBox ? '已关闭' : '已开启'}`);
        $('.hide-sponsor-shop').html(`赞助商店：${conf.hideSponsorShop ? '已关闭' : '已开启'}`);
        $('.show-fate-month').html(`命运之月弹窗：${conf.showFateMonth ? '已开启' : '已关闭'}`)
        $('.hide-ask-win').html(`常见问题：${conf.hideAskWin ? '已关闭' : '已开启'}`);

        $('.init-dlc').each(function(){
            const attr = $(this).attr('data-attr');
            const name = $(this).attr('data-name');
            $(this).html(`${name}：${data[attr] ? '已开启' : '已关闭'}`);
        })
    }catch (e) {

        console.error(e)
    }

    $('.system').show();
})

//获取网络时间
function getNetDate(callback = function(date){}){
    try{
        const httpRequest = new XMLHttpRequest();
        httpRequest.open('GET', 'https://quan.suning.com/getSysTime.do', true);
        httpRequest.setRequestHeader("Content-type","application/x-www-form-urlencoded");
        httpRequest.send(null);
        httpRequest.timeout = 3000;

        httpRequest.onreadystatechange = function () {
            if (httpRequest.readyState == 4) {
                if(httpRequest.status == 200){
                    callback(new Date(JSON.parse(httpRequest.responseText).sysTime2));
                }else{
                    window.notNetDate = true;
                    callback(new Date());
                }
            }
        };
    }catch (e) {
        callback(new Date());
    }
}

let autoBattleId = null;
function callAutoBattle(id){
    if(!conf.keepAtk || id === 'auto'){
        return;
    }
    $('.stop-auto-battle').show();
    autoBattleId = setInterval(function () {
        const bagItemById = getBagItemById(id);
        if(!data.battleMonster || data.battleMonster.hp <= 0 || (bagItemById && bagItemById.endurance <= 0)){
            clearInterval(autoBattleId);
            autoBattleId = null;
            $('.stop-auto-battle').hide();
            renderBattlePage();
        }else if(isSeal(data) && bagItemById && bagItemById.name.startsWith('技能石')){
            addBattleLog('被封印时停止技能连续攻击')
            clearInterval(autoBattleId);
            autoBattleId = null;
            $('.stop-auto-battle').hide();
            renderBattlePage();
        }else if(data.hp <= data.maxHp * 0.2){
            addBattleLog('少于20%血量停止连续攻击')
            clearInterval(autoBattleId);
            autoBattleId = null;
            $('.stop-auto-battle').hide();
            renderBattlePage();
        }else{
            battleClickWeapon(id)
        }
    }, 50);
}
$('.weapons').on({
    touchstart: function(e){
        callAutoBattle($(this).attr('data-id'))
    },
}, '.weapon');
$("body").on({
    mouseup: function(e){
        if (autoBattleId) {
            clearInterval(autoBattleId);
            autoBattleId = null;
            $('.stop-auto-battle').hide();
            renderBattlePage();
        }
    },
    touchmove: function(e){
        if (autoBattleId) {
            clearInterval(autoBattleId);
            autoBattleId = null;
            $('.stop-auto-battle').hide();
            renderBattlePage();
        }
    },
    touchend: function(e){
        if (autoBattleId) {
            clearInterval(autoBattleId);
            autoBattleId = null;
            $('.stop-auto-battle').hide();
            renderBattlePage();
        }
    },
});
