//神殿默认初始神性范围
const templeDefaultDivinity = [10, 50];
//低于多少神眷将会转变为神罚
const minGodFavor = -100;
//低于多少神眷神罚将开始导致怪物变异
const minGodFavorMonster = -1000;
//触发神战
const godBattle = -10000;
//神罚怪物，每-100点神眷，将产生多少属性的加强
const godMonsterAddAttr = {
    hp : 50,
    atk : 2,
    def : 1
};

//神灵类
function Deity(props = {}){
    for(const key in props){
        this[key] = props[key];
    }
}
Deity.prototype = {
    name : '神',//神名
    godFavor : 0,//神眷
    belief : 0,//信仰值
    hates : [],//憎恨的怪物,num代表需要击杀的数量,[{name : '怪物名', num : 10, godFavor : 10, belief : 10}]
    likes : [],//喜欢的物品,num代表需要献祭的数量,[{name : '物品名', num : 10,godFavor : 10, belief : 10}]
    sells : [],//出售的商品[{name : '商品名',godFavor : 0, belief : 100}]
};

//便捷创建神灵
function createDeity(name, props){
    return {[name] : new Deity({name, ...props})};
}

//便捷创建神殿
function createTemple(name, index){
    return {
        id : getNewId(),//神殿ID
        name : `${name}神殿`,//神殿名
        deity : name,//所属神灵名
        task : getTask(name, index),//神谕任务
        divinity : randomNum(templeDefaultDivinity[0],templeDefaultDivinity[1]),//初始神性
    };
}

//渲染神灵界面
function renderDeityTask(){
    $('.task-info').html('');
    const deitys = Object.keys(data.deitys);
    deitys.sort(function(a, b){
        return data.deitys[b].godFavor - data.deitys[a].godFavor
    })

    deitys.forEach(e => {
        let html = '<div class="deity-one">';
        html += `<h4>${e}</h4>`;
        html += `<div>神眷：${data.deitys[e].godFavor}&nbsp;&nbsp;&nbsp;&nbsp;信仰：${data.deitys[e].belief}</div>`;
        html += `<h4>神谕任务</h4>`;

        Object.keys(data.temples).forEach(index => {
            Object.keys(data.temples[index]).forEach(t => {
                if(data.temples[index][t] && data.temples[index][t].deity === e){
                    const temple = data.temples[index][t];
                    html += `<div>${data.maps[index].name}：${temple.task.name}<span class="to-cybernet" data-name="${temple.task.item || temple.task.monster}">🔍</span>(${temple.task.killed || getNumByBag(temple.task.item)}/${temple.task.num})</div>`;
                }
            })
        })
        html += `</div>`;
        $('.task-info').append(html);
    })
}

//获取神谕任务
function getTask(name, index){
    //获取神灵厌恶的怪物列表和喜欢的物品列表
    const {hates, likes} = deitys[name];
    //判断是否是击杀怪物的任务
    const needKill = checkMonthName('猎杀之月') ? true : checkMonthName('献祭之月') ? false : randomNum(1,100) <= 50;

    //击杀怪物
    if(needKill){
        //获取当前地图会出没的怪物集合
        const hasMonsterNames = [];
        terrainArray[data.maps[index].name].forEach(mapName => {
            //获取怪物配置
            const monsters = terrainElements[mapName] ? terrainElements[mapName].monster : {};
            //循环怪物配置
            for(const name in monsters){
                //符合等级的怪物放进配置里
                if(monsters[name].level >= data.maps[index].level[0] - (monsters[name].type ? 10 : 0) && monsters[name].level <= data.maps[index].level[1]){
                    hasMonsterNames.push(name);
                }
            }
        });
        //神厌恶且在本地图出现的怪物名集合
        const monsterNames = hates.map(e => e.name).filter(e => hasMonsterNames.indexOf(e) !== -1);
        //获取任务要求和奖励
        const monsterName = monsterNames.getRandom();
        const {num, godFavor, belief} = monsterName ? hates.filter(e => e.name === monsterName)[0] : {name : '无害小动物', num : 10, godFavor : 0, belief : 0};
        return {
            type : 'kill',
            name : `消灭${monsterName}`,//任务名
            monster : monsterName,//需要消灭的怪物
            num,//需要消灭的数量
            killed : 0,//已经击杀的数量
            godFavor,//奖励神眷值
            belief,//奖励信仰值
        }

    }else{//献祭物品
        //获取本地图可作为献祭物品
        let canSubmitItems = [];
        Object.keys(goods[data.maps[index].name]).forEach(e => {
            canSubmitItems = [...canSubmitItems,...goods[data.maps[index].name][e]];
        })
        const {name : needItem, num, godFavor, belief} = likes.filter(e => canSubmitItems.indexOf(e.name) !== -1).getRandom();
        return {
            type : 'get',
            name : `献祭${needItem}`,//任务名
            item : needItem,//需要献祭的物品
            num,//需要献祭的数量
            godFavor,//奖励神眷值
            belief,//奖励信仰值
        }

    }
}

//点击打开神殿
$('.body').on('click','.temple-map',function(){
    const deity = data.temples[data.worldIndex][data.location.join()].deity;
    if(data.deitys[deity].godFavor <= minGodFavor){
        const atk = parseInt(data.deitys[deity].godFavor / 100);
        data.hp += atk;
        winMsg(`${deity}降下了神罚，对你造成${-atk}伤害。`);
        renderBag();
        if(data.hp <= 0){
            addDieLog(`${deity}降下了神罚，导致你死亡`);
            die();
        }else{
            save();
        }
    }else{
        renderTemple();
        $('.temple').show();
    }
});

//渲染神殿
function renderTemple(){
    const temple = data.temples[data.worldIndex][data.location.join()];
    $('.temple>.win-close-title').html(`离开${temple.name}`);
    $('.divinity span').html(temple.divinity);
    $('.destroy span').html(parseInt(temple.divinity / 10));
}

//查看任务
$('.task').click(function(){
    const task = data.temples[data.worldIndex][data.location.join()].task;
    $('.task-name').html(`${task.name}<span class="to-cybernet" data-name="${task.item || task.monster}">🔍</span>`);
    if(task.type === 'kill'){
        $('.task-num').html(`数量：${task.killed}/${task.num}`);
    }else{
        $('.task-num').html(`数量：${getNumByBag(task.item, data.location.join() === data.world.home.location.join())}/${task.num}`);
    }
    $('.task-award').html(`奖励：神眷(${task.godFavor})，信仰(${task.belief})`);
    $('.task-win').show();
})

//查看商店
$('.shop').click(function(){
    renderDeityShop();
    $('.deity-shop').show();
})

//渲染商店
function renderDeityShop(){
    try{
        $('.deity-shop .buy').html('');
        const deity = data.temples[data.worldIndex][data.location.join()].deity;
        const sells = [
            ...deitys[deity].sells,
            {name : '心灵净化卷轴',godFavor : 0, belief : 10},
            {name : '心灵洗涤泉水',godFavor : 1000, belief : 500},
        ];

        $('.deity-shop .gold').html(`神眷：${data.deitys[deity].godFavor}&nbsp;&nbsp;&nbsp;信仰：${data.deitys[deity].belief}`);
        sells.forEach(e => {
            let html = '';
            html += `<div class="buy-good">`;
            html += `<div><div class="good-name">${e.name}<span class="to-cybernet" data-name="${e.name}">🔍</span></div><div class="merchant">要求神眷：${e.godFavor}</div></div>`;
            html += `<div><div class="price">单价：${e.belief}信仰</div><div class="buy-btns">`;
            html += `<button class="buy-deity-good-btn" data-godFavor="${e.godFavor}" data-belief="${e.belief}" data-name="${e.name}" data-num="1">买1个</button>`;
            html += `<button class="buy-deity-good-btn" data-godFavor="${e.godFavor}" data-belief="${e.belief}" data-name="${e.name}" data-num="10">买10个</button>`;
            html += `</div></div>`;
            $('.deity-shop .buy').append(html);
        })
    }catch (e) {
        console.error(e)
        msg('神恩商店渲染错误：' + e)
    }

}

//购买物品
$('.deity-shop .buy').on('click','.buy-deity-good-btn',function(){
    const deity = data.temples[data.worldIndex][data.location.join()].deity;
    const name = $(this).attr('data-name');
    const num = parseInt($(this).attr('data-num'));
    const belief = parseInt($(this).attr('data-belief'));
    const godFavor = parseInt($(this).attr('data-godFavor'));
    const need = belief * num;

    if(data.deitys[deity].godFavor < godFavor){
        winMsg('神眷不足',false,false)
        return;
    }
    if(data.deitys[deity].belief < need){
        winMsg('信仰不足',false,false)
        return;
    }

    winMsg(`购买了${name}*${num}，消耗${need}信仰`)
    data.deitys[deity].belief += need;
    addItem(name,num);
    renderDeityShop();
    renderBag();

    save();
});

//提交任务
$('.submit-task').click(function(){
    const task = data.temples[data.worldIndex][data.location.join()].task;
    const deity = data.temples[data.worldIndex][data.location.join()].deity;

    //判断任务是否完成
    let success = false;
    if(task.type === 'kill' && task.killed >= task.num){
        success = true;
    }else if(getNumByBag(task.item, data.location.join() === data.world.home.location.join()) >= task.num){
        //如果是在家，允许使用仓库中的物品
        if(data.location.join() === data.world.home.location.join()){
            //获取背包中的数量
            const bagNum = getNumByBag(task.item);
            //优先使用背包中的物品
            if(bagNum >= task.num){
                reduceItem(task.item, task.num);
            }else{
                if(bagNum > 0){
                    reduceItem(task.item, bagNum);
                }
                reduceItemByStore(task.item, task.num - bagNum);
            }

        }else{
            reduceItem(task.item, task.num)
        }
        success = true;
        renderBag();
    }

    //完成任务
    if(success){
        data.deitys[deity].godFavor += task.godFavor;
        data.deitys[deity].belief += task.belief;
        winMsg(`完成任务${task.name}，获得神眷(${task.godFavor})，信仰(${task.belief})`);

        //提高神性
        data.temples[data.worldIndex][data.location.join()].divinity++;
        renderTemple();

        //更换任务
        data.temples[data.worldIndex][data.location.join()].task = getTask(deity, data.worldIndex);
        $('.task-win').hide();

        save();
    }else{//未完成
        winMsg('你还未完成任务');
    }
})

//拒绝任务
$('.refuse').click(function(){
    if(!confirm('拒绝任务将扣除10神眷和10信仰，是否确认拒绝？')){
        return;
    }
    const deity = data.temples[data.worldIndex][data.location.join()].deity;
    //减少信仰值和神眷值
    data.deitys[deity].godFavor += 10000;
    data.deitys[deity].belief += 10000;

    winMsg(`你拒绝了${deity}的神谕，${deity}对你的神眷减少10，信仰减少10。`);
    //更换任务
    data.temples[data.worldIndex][data.location.join()].task = getTask(deity, data.worldIndex);
    $('.task-win').hide();

    //转为神罚
    if(data.deitys[deity].godFavor <= minGodFavor){
        winMsg(`你惹怒了${deity}，${deity}拒绝你进入${deity}神殿。`);
        $('.temple').hide();
    }

    save();
})

//摧毁神殿
$('.destroy').click(function(){
    if(data.atk < 1000){
        winMsg(`需要至少1000攻击才能摧毁神殿`, false, false)
        return;
    }
    if(confirm('该操作无法恢复，是否确认摧毁该神殿？')){
        const temple = data.temples[data.worldIndex][data.location.join()];
        const deity = data.temples[data.worldIndex][data.location.join()].deity;
        const num = parseInt(temple.divinity / 10);

        //获取神性结晶
        addItem('神性结晶', num);
        //减少神眷和信仰
        data.deitys[deity].godFavor = 0;
        data.deitys[deity].belief = 0;
        data.deitys[deity].godFavor += temple.divinity * 100;
        data.deitys[deity].belief += temple.divinity * 100;

        winMsg(`你摧毁了${deity}神殿，获得神性结晶*${num}。你惹怒了${deity}，${deity}拒绝你进入${deity}神殿。${deity}对你的神眷和信仰全部清空，并且减少${temple.divinity * 100}神眷和信仰值。`);

        //改变地图状态
        data.map[data.location[1]][data.location[0]] = '遗址';
        data.world.map[data.location[1]][data.location[0]] = '遗址';
        data.temples[data.worldIndex][data.location.join()] = null;
        renderMapElements();
        //渲染可前往的地图
        renderCanToMap();
        //渲染地图
        renderMap();

        $('.temple').hide();
        save();
    }
})

//神殿回城卷使用
function deityBack(deityName){
    if(data.inDungeon){
        winMsg(`在副本中不可返回神殿`);
        return false;
    }
    if(data.deitys[deityName].godFavor <= minGodFavor){
        const atk = parseInt(data.deitys[deityName].godFavor / 100);
        data.hp += atk;
        winMsg(`${deityName}降下了神罚，对你造成${-atk}伤害。`);
        renderBag();
        if(data.hp <= 0){
            addDieLog(`${deityName}降下了神罚，导致你死亡`);
            die();
        }
        return true;
    }else{
        const temples = Object.keys(data.temples[data.worldIndex]).filter(point => data.temples[data.worldIndex][point] && data.temples[data.worldIndex][point].deity === deityName);
        if(temples.length > 0){
            const str = temples[0].split(',');
            data.location = [parseInt(str[0]),parseInt(str[1])];
            //将当前坐标和四周坐标设置为已探索
            setShowMap();
            //渲染地图元素
            renderMapElements();
            //渲染可前往的地图
            renderCanToMap();
            //渲染地图
            renderMap();
            $('.item-use-win-modal').hide();
            $('.item-use-win').hide();
            $('.bag').hide();
            $('.home').hide();
            return true;
        }else{
            winMsg(`当前地图没有${deityName}神殿`);
            return false;
        }
    }
}

//神灵列表
const deitys = {
    ...createDeity('水之女神',{
        hates : [
            {name : '哥布林', num : 20, godFavor : 10, belief : 10},
            {name : '蜘蛛', num : 20, godFavor : 10, belief : 10},
            {name : '蝎子', num : 20, godFavor : 10, belief : 10},
            {name : '蠕虫', num : 20, godFavor : 15, belief : 15},
            {name : '鱼人', num : 20, godFavor : 15, belief : 15},
            {name : '哥布林巨人', num : 3, godFavor : 30, belief : 30},
            {name : '扭曲邪物', num : 1, godFavor : 50, belief : 50},

            {name : '哥布林英雄', num : 1, godFavor : 30, belief : 30},
            {name : '大哥布林', num : 20, godFavor : 20, belief : 20},
            {name : '洞穴人', num : 20, godFavor : 20, belief : 20},
            {name : '巨蝎', num : 15, godFavor : 20, belief : 20},
            {name : '食尸鬼', num : 15, godFavor : 20, belief : 20},
            {name : '火蛇', num : 15, godFavor : 25, belief : 25},
            {name : '游荡者', num : 3, godFavor : 50, belief : 50},
            {name : '扭曲的憎恨', num : 1, godFavor : 100, belief : 100},

            {name : '硬鳞鱼人', num : 20, godFavor : 20, belief : 20},
            {name : '毒刺海胆', num : 20, godFavor : 20, belief : 20},
            {name : '披甲鱼人', num : 15, godFavor : 20, belief : 20},
            {name : '蓝鳞海蛇', num : 15, godFavor : 20, belief : 20},
            {name : '鱼人斥候', num : 15, godFavor : 20, belief : 20},
            {name : '鱼人战士', num : 15, godFavor : 20, belief : 20},
            {name : '鱼人法师', num : 15, godFavor : 20, belief : 20},
            {name : '弄潮鱼人', num : 3, godFavor : 50, belief : 50},
            {name : '血肉幽灵船', num : 1, godFavor : 100, belief : 100},

            {name : '哥布林战士', num : 10, godFavor : 30, belief : 30},
            {name : '角蛇', num : 10, godFavor : 30, belief : 30},
            {name : '蝎尾狮', num : 10, godFavor : 40, belief : 40},
            {name : '半兽人', num : 10, godFavor : 40, belief : 40},
            {name : '强兽人', num : 10, godFavor : 40, belief : 40},

            {name : '重装哥布林', num : 10, godFavor : 40, belief : 40},
            {name : '哥布林骑士', num : 10, godFavor : 40, belief : 40},
            {name : '哥布林祭司', num : 10, godFavor : 40, belief : 40},
            {name : '刺蛇', num : 10, godFavor : 40, belief : 40},
            {name : '幽灵蛛', num : 10, godFavor : 50, belief : 50},
            {name : '死神蝎', num : 10, godFavor : 50, belief : 50},
            {name : '黄酮蟒', num : 10, godFavor : 50, belief : 50},
            {name : '氏族兽人', num : 10, godFavor : 50, belief : 50},

            {name : '骷髅战士', num : 10, godFavor : 50, belief : 50},
            {name : '骷髅法师', num : 10, godFavor : 50, belief : 50},
            {name : '活尸', num : 10, godFavor : 50, belief : 50},
            {name : '幽魂', num : 10, godFavor : 60, belief : 60},
            {name : '僵尸蟒', num : 10, godFavor : 60, belief : 60},
            {name : '血肉亡灵', num : 10, godFavor : 60, belief : 60},
            {name : '血肉屠夫', num : 10, godFavor : 60, belief : 60},
            {name : '尸巫', num : 10, godFavor : 70, belief : 70},
            {name : '吸血鬼', num : 10, godFavor : 70, belief : 70},
            {name : '石像鬼', num : 10, godFavor : 70, belief : 70},
            {name : '骷髅骑士', num : 10, godFavor : 80, belief : 80},
            {name : '血魔', num : 10, godFavor : 80, belief : 80},
            {name : '怨灵', num : 10, godFavor : 80, belief : 80},

            {name : '骷髅剑士', num : 10, godFavor : 80, belief : 80},
            {name : '巨魔骷髅', num : 10, godFavor : 80, belief : 80},
            {name : '亡灵蠕虫', num : 10, godFavor : 90, belief : 90},
            {name : '鲜血巨魔', num : 10, godFavor : 90, belief : 90},

            {name : '隐秘猎人', num : 10, godFavor : 100, belief : 100},
            {name : '阴影杀手', num : 10, godFavor : 100, belief : 100},
            {name : '水生战兽', num : 10, godFavor : 100, belief : 100},
            {name : '白鳞水蟒', num : 10, godFavor : 100, belief : 100},
            {name : '鹰身女妖', num : 10, godFavor : 110, belief : 110},
            {name : '飓风龙兽', num : 10, godFavor : 110, belief : 110},

            {name : '怒焰亚龙', num : 10, godFavor : 120, belief : 120},
            {name : '冰霜亚龙', num : 10, godFavor : 120, belief : 120},
            {name : '毒液亚龙', num : 10, godFavor : 120, belief : 120},
            {name : '黑暗亚龙', num : 10, godFavor : 120, belief : 120},
            {name : '红龙', num : 10, godFavor : 130, belief : 130},
            {name : '蓝龙', num : 10, godFavor : 130, belief : 130},
            {name : '绿龙', num : 10, godFavor : 130, belief : 130},
            {name : '黑龙', num : 10, godFavor : 130, belief : 130},

            {name : '无害小动物', num : 10, godFavor : 0, belief : 0},
        ],
        likes : [
            {name : '草药', num : 20, godFavor : 10, belief : 10},
            {name : '魔晶石', num : 20, godFavor : 20, belief : 20},
            {name : '贝壳', num : 10, godFavor : 10, belief : 10},
            {name : '螃蟹', num : 10, godFavor : 10, belief : 10},
            {name : '粘液', num : 30, godFavor : 10, belief : 10},
            {name : '羽毛', num : 10, godFavor : 10, belief : 10},
            {name : '变异核心', num : 20, godFavor : 30, belief : 30},
            {name : '哥布林之心', num : 3, godFavor : 20, belief : 20},
            {name : '哥布林魔晶', num : 1, godFavor : 20, belief : 20},
            {name : '神性结晶', num : 1, godFavor : 50, belief : 50},
            {name : '神性液体', num : 1, godFavor : 20, belief : 20},

            {name : '红水晶', num : 20, godFavor : 20, belief : 20},
            {name : '纯净水晶', num : 10, godFavor : 20, belief : 20},
            {name : '蘑菇', num : 30, godFavor : 10, belief : 10},
            {name : '魔核', num : 10, godFavor : 10, belief : 10},
            {name : '火结晶', num : 10, godFavor : 20, belief : 20},
            {name : '灵魂残片', num : 20, godFavor : 10, belief : 10},
            {name : '微光蘑菇', num : 20, godFavor : 10, belief : 10},
            {name : '灵魂石', num : 3, godFavor : 20, belief : 20},
            {name : '遗忘之魂', num : 1, godFavor : 20, belief : 20},
            {name : '神性魂石', num : 1, godFavor : 50, belief : 50},
            {name : '远古精魄', num : 1, godFavor : 50, belief : 50},

            {name : '珊瑚', num : 20, godFavor : 10, belief : 10},
            {name : '珍珠', num : 5, godFavor : 10, belief : 10},
            {name : '蓝水晶', num : 20, godFavor : 10, belief : 10},
            {name : '海龟壳', num : 10, godFavor : 10, belief : 10},
            {name : '蓝鳞蛇皮', num : 10, godFavor : 10, belief : 10},
            {name : '元素水', num : 10, godFavor : 10, belief : 10},
            {name : '水之核', num : 10, godFavor : 10, belief : 10},
            {name : '潮汐之眼', num : 1, godFavor : 20, belief : 20},
            {name : '海浪之鳞', num : 1, godFavor : 20, belief : 20},
            {name : '潮汐之心', num : 1, godFavor : 20, belief : 20},
            {name : '蓝光鳞片', num : 1, godFavor : 30, belief : 30},
            {name : '光辉鱼眼', num : 1, godFavor : 50, belief : 50},
            {name : '冰凉鱼油', num : 1, godFavor : 50, belief : 50},

            {name : '紫晶', num : 20, godFavor : 20, belief : 20},
            {name : '血肉花蕊', num : 10, godFavor : 20, belief : 20},
            {name : '史莱姆精华', num : 20, godFavor : 10, belief : 10},
            {name : '史莱姆核心', num : 20, godFavor : 20, belief : 20},
            {name : '疾风鹰羽', num : 10, godFavor : 10, belief : 10},
            {name : '柔韧根茎', num : 10, godFavor : 10, belief : 10},
            {name : '雷角', num : 10, godFavor : 20, belief : 20},
            {name : '灵魂晶粉', num : 10, godFavor : 20, belief : 20},
            {name : '食人花汁', num : 10, godFavor : 20, belief : 20},
            {name : '紫晶之核', num : 3, godFavor : 50, belief : 50},
            {name : '神性紫晶', num : 1, godFavor : 50, belief : 50},
            {name : '神性枢纽', num : 1, godFavor : 50, belief : 50},

            {name : '强酸粘液', num : 15, godFavor : 30, belief : 30},
            {name : '酸性软皮', num : 15, godFavor : 30, belief : 30},
            {name : '荒芜之核', num : 10, godFavor : 30, belief : 30},
            {name : '亚龙鳞', num : 10, godFavor : 30, belief : 30},
            {name : '大地之牙', num : 10, godFavor : 30, belief : 30},
            {name : '大地之皮', num : 10, godFavor : 30, belief : 30},
            {name : '五彩蛟麟', num : 3, godFavor : 50, belief : 50},
            {name : '魔蛟精血', num : 3, godFavor : 50, belief : 50},
            {name : '魔蛟骨髓', num : 3, godFavor : 50, belief : 50},
            {name : '残破徽章', num : 1, godFavor : 50, belief : 50},
            {name : '龙晶', num : 1, godFavor : 50, belief : 50},
            {name : '幼龙鳞', num : 1, godFavor : 50, belief : 50},

            {name : '幽魂碎片', num : 10, godFavor : 50, belief : 50},
            {name : '幽魂核心', num : 10, godFavor : 50, belief : 50},
            {name : '尸巫法杖', num : 10, godFavor : 60, belief : 60},
            {name : '亡灵法核', num : 10, godFavor : 60, belief : 60},
            {name : '吸血鬼牙', num : 10, godFavor : 60, belief : 60},
            {name : '血族翅膀', num : 10, godFavor : 60, belief : 60},
            {name : '血魔之心', num : 10, godFavor : 70, belief : 70},
            {name : '怨恨之魂', num : 10, godFavor : 70, belief : 70},
            {name : '诅咒核心', num : 10, godFavor : 70, belief : 70},

            {name : '战士徽章', num : 10, godFavor : 70, belief : 70},
            {name : '赐福铠甲', num : 10, godFavor : 70, belief : 70},
            {name : '魔法宝石', num : 10, godFavor : 80, belief : 80},
            {name : '法师帽', num : 10, godFavor : 80, belief : 80},
            {name : '附魔箭矢', num : 10, godFavor : 80, belief : 80},
            {name : '赐福长枪', num : 10, godFavor : 90, belief : 90},
            {name : '骑士徽章', num : 10, godFavor : 90, belief : 90},
            {name : '元素印记', num : 10, godFavor : 90, belief : 90},
            {name : '女巫徽章', num : 10, godFavor : 90, belief : 90},
            {name : '精铁断剑', num : 1, godFavor : 100, belief : 100},
            {name : '大剑士徽章', num : 1, godFavor : 100, belief : 100},

            {name : '狂信徽章', num : 10, godFavor : 90, belief : 90},
            {name : '信仰雕像', num : 10, godFavor : 90, belief : 90},
            {name : '白鳞', num : 10, godFavor : 100, belief : 100},
            {name : '白皮', num : 10, godFavor : 100, belief : 100},
            {name : '女妖翅膀', num : 10, godFavor : 110, belief : 110},
            {name : '飓风魔核', num : 10, godFavor : 110, belief : 110},
            {name : '神圣长袍', num : 10, godFavor : 110, belief : 110},

            {name : '烈焰亚龙牙', num : 10, godFavor : 120, belief : 120},
            {name : '冰霜亚龙牙', num : 10, godFavor : 120, belief : 120},
            {name : '剧毒亚龙牙', num : 10, godFavor : 120, belief : 120},
            {name : '黑暗亚龙牙', num : 10, godFavor : 120, belief : 120},
            {name : '红龙魂', num : 10, godFavor : 130, belief : 130},
            {name : '蓝龙魂', num : 10, godFavor : 130, belief : 130},
            {name : '绿龙魂', num : 10, godFavor : 130, belief : 130},
            {name : '黑龙魂', num : 10, godFavor : 130, belief : 130},

            {name : '无害小动物', num : 10, godFavor : 0, belief : 0},
        ],
        sells : [
            {name : '水之女神回城卷',godFavor : 0, belief : 1},
            {name : '时空粉尘',godFavor : 0, belief : 10},
            {name : '时空碎片',godFavor : 0, belief : 50},
            {name : '哥布林杀手',godFavor : 0, belief : 200},
            {name : '弱小联合',godFavor : 0, belief : 500},
            {name : '神血',godFavor : 2000, belief : 1000},
            {name : '转生卷轴：水之神裔',godFavor : 2000, belief : 200},
            {name : '永恒珠宝',godFavor : 3000, belief : 1000},
            {name : '位面坐标：宝石',godFavor : 0, belief : 100},
        ],
    }),
    ...createDeity('狩猎之神',{
        hates : [
            {name : '狼', num : 20, godFavor : 10, belief : 10},
            {name : '巨鼠', num : 20, godFavor : 10, belief : 10},
            {name : '蜘蛛', num : 20, godFavor : 10, belief : 10},
            {name : '巨蟒', num : 20, godFavor : 10, belief : 10},
            {name : '蝎子', num : 20, godFavor : 10, belief : 10},
            {name : '蠕虫', num : 20, godFavor : 15, belief : 15},
            {name : '巨蜥', num : 20, godFavor : 15, belief : 15},
            {name : '食人鱼', num : 20, godFavor : 10, belief : 10},
            {name : '狮鹫', num : 15, godFavor : 10, belief : 10},
            {name : '鳄龟', num : 15, godFavor : 10, belief : 10},

            {name : '蝙蝠', num : 20, godFavor : 10, belief : 10},
            {name : '猩红蝾螈', num : 15, godFavor : 10, belief : 10},
            {name : '巨蝎', num : 15, godFavor : 10, belief : 10},
            {name : '火蛇', num : 15, godFavor : 20, belief : 20},

            {name : '巨钳蟹', num : 20, godFavor : 10, belief : 10},
            {name : '毒刺海胆', num : 20, godFavor : 10, belief : 10},
            {name : '铁甲海龟', num : 15, godFavor : 10, belief : 10},
            {name : '蓝鳞海蛇', num : 10, godFavor : 10, belief : 10},
            {name : '独角海象', num : 1, godFavor : 40, belief : 40},

            {name : '疾风狼', num : 10, godFavor : 30, belief : 30},
            {name : '暗影猎豹', num : 10, godFavor : 30, belief : 30},
            {name : '剑齿虎', num : 10, godFavor : 40, belief : 40},
            {name : '角蛇', num : 10, godFavor : 40, belief : 40},
            {name : '疾风猎鹰', num : 10, godFavor : 40, belief : 40},
            {name : '雷角马', num : 10, godFavor : 40, belief : 40},
            {name : '蝎尾狮', num : 10, godFavor : 40, belief : 40},

            {name : '刺蛇', num : 10, godFavor : 40, belief : 40},
            {name : '幽灵蛛', num : 10, godFavor : 40, belief : 40},
            {name : '死神蝎', num : 10, godFavor : 40, belief : 40},
            {name : '荒原狼', num : 10, godFavor : 40, belief : 40},
            {name : '厚皮蜥', num : 10, godFavor : 40, belief : 40},
            {name : '迅猛龙', num : 10, godFavor : 50, belief : 50},
            {name : '荒龙', num : 10, godFavor : 50, belief : 50},
            {name : '青牙兽', num : 10, godFavor : 50, belief : 50},
            {name : '黄酮蟒', num : 10, godFavor : 50, belief : 50},
            {name : '地牙野猪', num : 10, godFavor : 50, belief : 50},
            {name : '大地熊', num : 10, godFavor : 50, belief : 50},
            {name : '五彩魔蛟', num : 3, godFavor : 80, belief : 80},
            {name : '荒芜亚龙', num : 1, godFavor : 100, belief : 100},
            {name : '绿龙幼崽', num : 1, godFavor : 120, belief : 120},

            {name : '骷髅战士', num : 10, godFavor : 50, belief : 50},
            {name : '活尸', num : 10, godFavor : 50, belief : 50},
            {name : '僵尸蟒', num : 10, godFavor : 60, belief : 60},
            {name : '血肉亡灵', num : 10, godFavor : 60, belief : 60},
            {name : '血肉屠夫', num : 10, godFavor : 60, belief : 60},
            {name : '吸血鬼', num : 10, godFavor : 70, belief : 70},
            {name : '石像鬼', num : 10, godFavor : 70, belief : 70},
            {name : '血魔', num : 10, godFavor : 80, belief : 80},

            {name : '骷髅剑士', num : 10, godFavor : 80, belief : 80},
            {name : '巨魔骷髅', num : 10, godFavor : 80, belief : 80},
            {name : '亡灵蠕虫', num : 10, godFavor : 90, belief : 90},
            {name : '鲜血巨魔', num : 10, godFavor : 90, belief : 90},

            {name : '水生战兽', num : 10, godFavor : 100, belief : 100},
            {name : '白鳞水蟒', num : 10, godFavor : 100, belief : 100},
            {name : '鹰身女妖', num : 10, godFavor : 110, belief : 110},
            {name : '飓风龙兽', num : 10, godFavor : 110, belief : 110},

            {name : '怒焰亚龙', num : 10, godFavor : 120, belief : 120},
            {name : '冰霜亚龙', num : 10, godFavor : 120, belief : 120},
            {name : '毒液亚龙', num : 10, godFavor : 120, belief : 120},
            {name : '黑暗亚龙', num : 10, godFavor : 120, belief : 120},
            {name : '红龙', num : 10, godFavor : 130, belief : 130},
            {name : '蓝龙', num : 10, godFavor : 130, belief : 130},
            {name : '绿龙', num : 10, godFavor : 130, belief : 130},
            {name : '黑龙', num : 10, godFavor : 130, belief : 130},
        ],
        likes : [
            {name : '草药', num : 20, godFavor : 10, belief : 10},
            {name : '尖牙', num : 20, godFavor : 10, belief : 10},
            {name : '利爪', num : 20, godFavor : 10, belief : 10},
            {name : '兽皮', num : 20, godFavor : 10, belief : 10},
            {name : '蜘蛛丝', num : 20, godFavor : 10, belief : 10},
            {name : '毒囊', num : 20, godFavor : 10, belief : 10},
            {name : '蛇胆', num : 20, godFavor : 10, belief : 10},
            {name : '甲壳', num : 20, godFavor : 10, belief : 10},
            {name : '羽毛', num : 20, godFavor : 10, belief : 10},

            {name : '硬皮', num : 20, godFavor : 20, belief : 20},
            {name : '硬壳', num : 20, godFavor : 20, belief : 20},
            {name : '兽骨', num : 20, godFavor : 20, belief : 20},
            {name : '魔核', num : 10, godFavor : 20, belief : 20},

            {name : '螃蟹壳', num : 20, godFavor : 20, belief : 20},
            {name : '巨大蟹钳', num : 20, godFavor : 20, belief : 20},
            {name : '海胆毒刺', num : 20, godFavor : 20, belief : 20},
            {name : '海胆皮', num : 20, godFavor : 20, belief : 20},
            {name : '海龟壳', num : 20, godFavor : 20, belief : 20},
            {name : '蓝鳞蛇皮', num : 10, godFavor : 20, belief : 20},
            {name : '海象角', num : 1, godFavor : 30, belief : 30},
            {name : '海象皮', num : 1, godFavor : 30, belief : 30},
            {name : '海象牙', num : 1, godFavor : 30, belief : 30},

            {name : '魔兽骨', num : 20, godFavor : 20, belief : 20},
            {name : '魔兽皮', num : 20, godFavor : 20, belief : 20},
            {name : '魔兽爪', num : 20, godFavor : 20, belief : 20},
            {name : '暗影血肉', num : 10, godFavor : 20, belief : 20},
            {name : '虎皮', num : 10, godFavor : 20, belief : 20},
            {name : '角蛇的角', num : 10, godFavor : 20, belief : 20},
            {name : '疾风鹰羽', num : 10, godFavor : 20, belief : 20},
            {name : '雷角', num : 10, godFavor : 30, belief : 30},
            {name : '蝎尾狮的尾巴', num : 10, godFavor : 30, belief : 30},

            {name : '骨刺', num : 10, godFavor : 30, belief : 30},
            {name : '毒牙', num : 10, godFavor : 30, belief : 30},
            {name : '幽灵蛛丝', num : 10, godFavor : 30, belief : 30},
            {name : '幽灵毒囊', num : 10, godFavor : 30, belief : 30},
            {name : '重壳', num : 10, godFavor : 30, belief : 30},
            {name : '巨爪', num : 10, godFavor : 30, belief : 30},
            {name : '巨牙', num : 10, godFavor : 30, belief : 30},
            {name : '重皮', num : 10, godFavor : 30, belief : 30},
            {name : '荒芜之核', num : 10, godFavor : 30, belief : 30},
            {name : '亚龙鳞', num : 10, godFavor : 30, belief : 30},
            {name : '青牙', num : 10, godFavor : 30, belief : 30},
            {name : '青牙兽皮', num : 10, godFavor : 30, belief : 30},
            {name : '黄酮鳞', num : 10, godFavor : 30, belief : 30},
            {name : '黄酮胆', num : 10, godFavor : 40, belief : 40},
            {name : '大地之牙', num : 10, godFavor : 40, belief : 40},
            {name : '大地之皮', num : 10, godFavor : 40, belief : 40},
            {name : '五彩蛟麟', num : 5, godFavor : 80, belief : 80},
            {name : '魔蛟精血', num : 5, godFavor : 80, belief : 80},
            {name : '魔蛟骨髓', num : 5, godFavor : 80, belief : 80},
            {name : '荒芜核心', num : 3, godFavor : 100, belief : 100},
            {name : '荒芜麟爪', num : 3, godFavor : 100, belief : 100},
            {name : '龙晶', num : 1, godFavor : 120, belief : 120},
            {name : '幼龙鳞', num : 3, godFavor : 120, belief : 120},
            {name : '幼龙皮', num : 3, godFavor : 120, belief : 120},

            {name : '幽魂碎片', num : 10, godFavor : 50, belief : 50},
            {name : '幽魂核心', num : 10, godFavor : 50, belief : 50},
            {name : '尸巫法杖', num : 10, godFavor : 60, belief : 60},
            {name : '亡灵法核', num : 10, godFavor : 60, belief : 60},
            {name : '吸血鬼牙', num : 10, godFavor : 60, belief : 60},
            {name : '血族翅膀', num : 10, godFavor : 60, belief : 60},
            {name : '血魔之心', num : 10, godFavor : 70, belief : 70},
            {name : '怨恨之魂', num : 10, godFavor : 70, belief : 70},
            {name : '诅咒核心', num : 10, godFavor : 70, belief : 70},

            {name : '精密骨骼', num : 10, godFavor : 70, belief : 70},
            {name : '骨质大剑', num : 10, godFavor : 70, belief : 70},
            {name : '巨魔骨矛', num : 10, godFavor : 70, belief : 70},
            {name : '巨魔颅骨', num : 10, godFavor : 70, belief : 70},
            {name : '灰白的皮', num : 10, godFavor : 80, belief : 80},
            {name : '枯朽粘液', num : 10, godFavor : 80, belief : 80},
            {name : '鲜血图腾', num : 10, godFavor : 90, belief : 90},
            {name : '鲜血骨爪', num : 10, godFavor : 90, belief : 90},
            {name : '精铁断剑', num : 1, godFavor : 100, belief : 100},
            {name : '大剑士徽章', num : 1, godFavor : 100, belief : 100},

            {name : '猎人陷阱', num : 10, godFavor : 100, belief : 100},
            {name : '鲜血毛皮', num : 10, godFavor : 100, belief : 100},
            {name : '浸水皮', num : 10, godFavor : 100, belief : 100},
            {name : '战兽爪', num : 10, godFavor : 100, belief : 100},
            {name : '白鳞', num : 10, godFavor : 100, belief : 100},
            {name : '白皮', num : 10, godFavor : 100, belief : 100},
            {name : '女妖手爪', num : 10, godFavor : 110, belief : 110},
            {name : '女妖翅膀', num : 10, godFavor : 110, belief : 110},
            {name : '飓风魔核', num : 10, godFavor : 110, belief : 110},
            {name : '飓风魔皮', num : 10, godFavor : 110, belief : 110},

            {name : '烈焰亚龙牙', num : 10, godFavor : 120, belief : 120},
            {name : '冰霜亚龙牙', num : 10, godFavor : 120, belief : 120},
            {name : '剧毒亚龙牙', num : 10, godFavor : 120, belief : 120},
            {name : '黑暗亚龙牙', num : 10, godFavor : 120, belief : 120},
            {name : '红龙魂', num : 10, godFavor : 130, belief : 130},
            {name : '蓝龙魂', num : 10, godFavor : 130, belief : 130},
            {name : '绿龙魂', num : 10, godFavor : 130, belief : 130},
            {name : '黑龙魂', num : 10, godFavor : 130, belief : 130},
        ],
        sells : [
            {name : '狩猎之神回城卷',godFavor : 0, belief : 1},
            {name : '时空粉尘',godFavor : 0, belief : 10},
            {name : '时空碎片',godFavor : 0, belief : 50},
            {name : '疯狂撕咬',godFavor : 0, belief : 1500},
            {name : '灵魂外壳',godFavor : 0, belief : 3000},
            {name : '神血',godFavor : 2000, belief : 1000},
            {name : '永恒珠宝',godFavor : 3000, belief : 1000},
            {name : '转生卷轴：狩猎神裔',godFavor : 2000, belief : 200},
            {name : '位面坐标：宝石',godFavor : 0, belief : 100},
        ],
    }),
}

Object.keys(newMapElements.deity).forEach(e => {
    deitys[e].hates = [...deitys[e].hates, ...newMapElements.deity[e].hates];
    deitys[e].likes = [...deitys[e].likes, ...newMapElements.deity[e].likes];
})
