//设定
const stockCfg = {
    金币 : {
        close : 100,//价位低于多少直接清算
        //初始化配置
        init : {
            //低价位股票
            low : {
                names : ['海产商会','森林木业'],//发行公司名称
                start : [200, 700]//初始价位
            },
            //中价位股票
            center : {
                names : ['紫晶商行','部落联盟','地底矿业','巫师制药','龙脉零售'],
                start : [1000, 10000]
            },
            //高价位股票
            high : {
                names : ['神国集团','时空财团','位面商会'],
                start : [20000, 50000]
            },
        },
    },
    赞助点数 : {
        close : 1,
        init : {
            low : {
                names : ['海产商会','森林木业'],
                start : [2, 7]
            },
            center : {
                names : ['紫晶商行','部落联盟','地底矿业','巫师制药','龙脉零售'],
                start : [10, 100]
            },
            high : {
                names : ['神国集团','时空财团','位面商会'],
                start : [200, 500]
            },
        },
    }
}

//初始化股市
function initStock(){
    if(!data.stock){
        data.stock = {
            day : data.date.format('yyyy-MM-dd'),
            count : {},
        };
        Object.keys(stockCfg).forEach(e => {
            data.stock.count[e] = {
                buyAllMoney : 0,//总共花了多少钱购买股票
                hasValue : 0,//持有点券的总价值
            };
            data.stock[e] = {};
            Object.keys(stockCfg[e].init).forEach(type => {
                const {names, start} = stockCfg[e].init[type];
                names.forEach(name => {
                    const price = parseFloat2Fiexd(randomNum(start[0] * 100, start[1] * 100) / 100);
                    data.stock[e][name] = {
                        price,
                        max : price,
                        min : price,
                        has : 0,
                        cost : 0,
                        float : 0,
                    };
                })
            })
        })
    }
}

//股市变动
function changeStock(){
    const day = data.date.format('yyyy-MM-dd');
    if(data.stock.day !== day){
        data.stock.day = day;
        let closeStocks = [];

        Object.keys(stockCfg).forEach(e => {
            const closePrice = stockCfg[e].close;
            Object.keys(data.stock[e]).forEach(name => {
                const changeNum = randomNum(-1000,1000);//随机跌10%到涨10%
                const now = data.stock[e][name];
                let price = parseFloat2Fiexd(now.price * (1 + parseFloat2Fiexd(changeNum / 10000)));

                if(price < closePrice){
                    const {start} = stockCfg[e].init[Object.keys(stockCfg[e].init).filter(f => stockCfg[e].init[f].names.includes(name))[0]];
                    const newPrice = parseFloat2Fiexd(randomNum(start[0] * 100, start[1] * 100) / 100);

                    //强制卖出所有
                    if(now.has > 0){
                        closeStocks.push(e+"："+name)
                        price = parseFloat2Fiexd(price * 0.99)
                        if(e === '金币'){
                            data.gold += now.has * price;
                            data.gold = parseFloat2Fiexd(data.gold);
                        }else{
                            data.sponsorPoint += now.has * price;
                            data.sponsorPoint = parseFloat2Fiexd(data.sponsorPoint);
                        }

                    }
                    data.stock[e][name] = {
                        price : newPrice,
                        max : newPrice,
                        min : newPrice,
                        has : 0,
                        cost : 0,
                        float : 0,
                    };
                }else{
                    data.stock[e][name] = {
                        ...now,
                        float : parseFloat2Fiexd(changeNum / 100),
                        price,
                        min : now.min > price ? price : now.min,
                        max : now.max < price ? price : now.max,
                    }
                }
            })
        })

        if(closeStocks.length > 0){
            $('.alert-win').show().find('.alert-win-title').html(`你持有的点券[${closeStocks.join('，')}]因为价值太低被迫破产清算，已强制卖出你持有的所有该点券，该点券已被其他财团重新接手并重置`)
        }
    save();
    }
}

//渲染股市
function renderStock(){
    initStock();
    const $stock = $('.stock-body');
    $stock.html('');
    const type = $('.stock-win .tab-head.active').html();

    //初始化价统计信息
    data.stock.count[type].hasValue = 0;

    //渲染所有股票
    Object.keys(data.stock[type]).sort((a,b) => data.stock[type][b].has - data.stock[type][a].has).forEach(e => {
        const {price, min, max, has, cost, float} = data.stock[type][e];
        //计算当前盈亏
        const stateNum = parseFloat2Fiexd(price * has - (cost * has));

        let html = '';
        html += '';
        html += `<div class="stock">`;
        html += `   <div class="stock-info">`;
        html += `       <div class="stock-info-one-name">${e}</div>`;
        html += `       <div class="stock-info-one">`;
        html += `           <div class="stock-info-one-title">单价</div>`;
        html += `           <div class="stock-info-one-body">${price}</div>`;
        html += `       </div>`;
        html += `       <div class="stock-info-one">`;
        html += `           <div class="stock-info-one-title">平均成本</div>`;
        html += `           <div class="stock-info-one-body">${cost}</div>`;
        html += `       </div>`;
        html += `       <div class="stock-info-one">`;
        html += `           <div class="stock-info-one-title">浮动</div>`;
        html += `           <div class="stock-info-one-body ${float > 0 ? 'stock-up' : float < 0 ? 'stock-down' : ''}">${float}%</div>`;
        html += `       </div>`;
        html += `       <div class="stock-info-one">`;
        html += `           <div class="stock-info-one-title">历史最低价</div>`;
        html += `           <div class="stock-info-one-body">${min}</div>`;
        html += `       </div>`;
        html += `       <div class="stock-info-one">`;
        html += `           <div class="stock-info-one-title">历史最高价</div>`;
        html += `           <div class="stock-info-one-body">${max}</div>`;
        html += `       </div>`;
        html += `       <div class="stock-info-one">`;
        html += `           <div class="stock-info-one-title">持有</div>`;
        html += `           <div class="stock-info-one-body">${has}</div>`;
        html += `       </div>`;
        html += `       <div class="stock-info-one">`;
        html += `           <div class="stock-info-one-title">当前盈利</div>`;
        html += `           <div class="stock-info-one-body ${stateNum > 0 ? 'stock-up' : stateNum < 0 ? 'stock-down' : ''}">${stateNum}</div>`;
        html += `       </div>`;
        html += `   </div>`;
        html += `   <div class="stock-btns">`;
        html += `      <div class="stock-btn" data-type="buy" data-name="${e}">购买</div>`;
        html += `      <div class="stock-btn" data-type="sell" data-name="${e}">出售</div>`;
        html += `   </div>`;
        html += `</div>`;

        $stock.append(html);
        data.stock.count[type].hasValue += parseFloat2Fiexd(has * price);
    })

    //渲染统计信息
    data.stock.count[type].hasValue = parseFloat2Fiexd(data.stock.count[type].hasValue);
    const {buyAllMoney, hasValue} = data.stock.count[type];
    $('#stock-type').html(type);
    $('#stock-type-money').html(type === '金币' ? data.gold || 0 : data.sponsorPoint || 0);
    $('#stock-type-value').html(hasValue);
    const allPal = parseFloat2Fiexd(hasValue - buyAllMoney);
    const allPalPer = parseFloat2Fiexd(buyAllMoney !== 0 ? allPal / buyAllMoney * 100 : 0);
    $('#stock-type-all-pal').html(`${allPal > 0 ? '+' : ''}${allPal}`);

    $('.stock-win').show();
}

//交易
$('.stock-win').on('click', '.stock-btn', function(){
    const type = $(this).attr('data-type');//买卖
    const name = $(this).attr('data-name');//股票名
    const money = $('.stock-win .tab-head.active').html();//货币

    let {price, has} = data.stock[money][name];
    if(type === 'buy'){//买
        $('.deal-type').html('购买')
        price = parseFloat2Fiexd(price * 1.01)
    }else{//卖
        $('.deal-type').html('出售')
        price = parseFloat2Fiexd(price * 0.99)
    }

    $('#deal-ok').attr({
        'data-type' : type,
        'data-name' : name,
    });
    $('#deal-name').html(name);
    $('.deal-money').html(money);
    $('#deal-has-money').html(money === '金币' ? data.gold || 0 : data.sponsorPoint || 0);
    $('#deal-has').html(has);
    $('#deal-price').html(price);
    $('#deal-count-price').html(0);
    $('#deal-count').html(0);

    $('.stock-deal-win-modal').show();
    $('.stock-deal-win').show();
})
$('.stock-deal-win .win-close-title').click(function(){
    $('.stock-deal-win-modal').hide();
})

//加减交易数值
$('.deal-btn').click(function(){
    const type = $('#deal-ok').attr('data-type');//买卖
    let num = $(this).attr('data-num');//增加的数量
    const money = $('.stock-win .tab-head.active').html();//货币
    const name = $('#deal-ok').attr('data-name');//股票名
    const hasMoney = money === '金币' ? data.gold || 0 : data.sponsorPoint || 0;//持有货币数量
    const count = parseInt($('#deal-count').html());//已有交易数量
    let {price, has} = data.stock[money][name];

    if(type === 'buy'){//买
        price = parseFloat2Fiexd(price * 1.01)
        const canBuy = parseInt(hasMoney / price) - count;//最多还可以购买的数量

        if(isNaN(num)){
            if(num === 'r1'){
                num = -(hasMoney / price);
            }else if(num === 'r2'){
                num = -parseInt((hasMoney / price) / 2);
            }else if(num === 'r5'){
                num = -parseInt((hasMoney / price) / 5);
            }else if(num === 'a1'){
                num = canBuy;
            }else if(num === 'a2'){
                num = parseInt(canBuy / 2);
            }else if(num === 'a5'){
                num = parseInt(canBuy / 5);
            }
        }else{
            num = parseInt(num)
        }

        if(canBuy < num){
            num = canBuy;
        }
        if(count < -num){
            num = -count;
        }
    }else{//卖
        price = parseFloat2Fiexd(price * 0.99)
        const canSell = has - count;//最多还可以出售的数量

        if(isNaN(num)){
            if(num === 'r1'){
                num = -has
            }else if(num === 'r2'){
                num = parseInt(-has / 2);
            }else if(num === 'r5'){
                num = parseInt(-has / 5);
            }else if(num === 'a1'){
                num = canSell;
            }else if(num === 'a2'){
                num = parseInt(canSell / 2);
            }else if(num === 'a5'){
                num = parseInt(canSell / 5);
            }
        }else{
            num = parseInt(num)
        }

        if(canSell < num){
            num = canSell;
        }
        if(num + count > has){
            num = has - count;
        }
        if(-num > count){
            num = -count;
        }
    }

    if(num){
        $('#deal-count-price').html(parseFloat2Fiexd((num + count) * price));
        $('#deal-count').html(num + count);
    }
});

//确认交易
$('#deal-ok').click(function(){
    const type = $('#deal-ok').attr('data-type');//买卖
    const name = $('#deal-ok').attr('data-name');//股票名
    const money = $('.stock-win .tab-head.active').html();//货币
    const allPrice = parseFloat2Fiexd($('#deal-count-price').html());//总价
    const count = parseInt($('#deal-count').html());//已有交易数量
    let price = data.stock[money][name].price;//单价
    const oldAllPrice = data.stock[money][name].has * data.stock[money][name].cost;//原本的总花费

    if(!allPrice){
        return;
    }

    if(type === 'buy'){//买
        price = parseFloat2Fiexd(price * 1.01)
        if(money === '金币'){
            data.gold -= allPrice;
            data.gold = parseFloat2Fiexd(data.gold);
        }else{
            data.sponsorPoint -= allPrice;
            data.sponsorPoint = parseFloat2Fiexd(data.sponsorPoint);
        }

        data.stock[money][name].has += count;
        data.stock.count[money].buyAllMoney += allPrice;
    }else{//卖
        price = parseFloat2Fiexd(price * 0.99)
        if(money === '金币'){
            data.gold += allPrice;
            data.gold = parseFloat2Fiexd(data.gold);
        }else{
            data.sponsorPoint += allPrice;
            data.sponsorPoint = parseFloat2Fiexd(data.sponsorPoint);
        }
        data.stock[money][name].has -= count;
        data.stock.count[money].buyAllMoney -= allPrice;
    }

    data.stock[money][name].cost = data.stock[money][name].has === 0 ? 0 : parseFloat2Fiexd(((count * price + oldAllPrice) / data.stock[money][name].has));
    $('.stock-deal-win .win-close-title').click();
    renderStock();
    save()
});
