/**
 * 便捷创建配方
 * @param items 需要的材料{'木材' : 2}
 * @param level 制作等级，为0则空手即可制作
 * @param need 必须使用指定设施制造,有need则无视level
 * @param getNum 制作一次获取的数量
 */
function createRecipesCfg(items, level = 0, need, getNum = 1){
    return {items, level : need ? 999 : level, need, getNum}
}

/**
 * 便捷创建可采集元素的方法
 * @param name 元素名
 * @param num 初始允许出现数量
 * @param maxNum 最大允许存在数量
 */
const terrainElement = function(name,num,maxNum){
    return {[name] : {num, maxNum}}
}

/**
 * 便捷创建锻造配方
 * @param items 需要的材料{'木材' : 2}
 * @param exp 要求熟练度
 */
function createForgeRecipesCfg(items, exp = 0){
    return {items, exp}
}

//元素类
function Element(props = {}){
    for(const key in props){
        this[key] = props[key];
    }
    this.maxEndurance = this.maxEndurance || this.endurance;
}

//元素类原型
Element.prototype = {
    //通用属性
    canUse : false,//是否可使用
    weight : 1,//重量
    endurance : 1,//耐久,耐久降到0则破坏
    maxEndurance : 0,//最大耐久度
    isWeapon : false,//是否是武器
    isArmor : false,//是否是护甲
    isShield : false,//是否是盾牌
    isInlay : false,//是否镶嵌物
    isAmulet : false,//是否护身符
    isMount : false,//是否坐骑
    saveOrder : false,//跳周目时是否保留
    isMeat : false,//是否肉类
    isBow : false,//是否是弓类武器
    isArrow : false,//是否是箭矢
    onlyOne : false,//是否是不可叠加的物品
    voidDust : 1,//可以分解出几个虚空粉尘
    text : null,//物品说明文本
    func : null,//使用时调用的方法，必须返回true才能成功使用物品
    gold : 1,//价值多少金币
    mopUp : false,//是否可以扫荡
    elemental : null, //元素属性，{}类型，可以有：金-攻击，木-吸血，水-耐久，火-生命，土-防御，极-减伤

    //可采集资源属性
    canGather : false,//是否可采集，仅用于标识给图鉴使用，无游戏功能
    time : 5,//采集时间（分钟）
    num : 1,//可采集次数
    day : -1,//生成天数,-1为不可生成
    needItem : [],//采集需要的物品['木镐','铁镐'],只要携带了列表中任意一个物品，即可采集。为空则没有要求
    canHand :  false,//是否可以空手采集，可以空手采集的物品必须是有需要工具的物品，如果不用工具，则采集数量固定变为-1~1，
    getItem : [],//采集一次可能获得的物品[{name : '物品名',minNum : 0, maxNum : 1}],minNum可以为负数，当随机数量在0以下时，都不会获取物品，所以可以minNum越小，获取到该物品的概率越低

    //可使用物品属性
    hp : 0,//增加生命值
    hunger : 0,//增加饥饿值
    water : 0,//增加水分
    useGetItem : [],//使用后可能获得的物品,配置参考 getItem
    battleUse : false,//是否可在战斗中使用
    getForgeRate : null,//学会锻造配方

    //坐骑属性
    weightUp : 0,//增加负重
    spd : 0,//增加速度
    tdi : false,//是否代替承受伤害，仅限坐骑类有效
    changeEquip(isEquip){},//切换装备与卸下
    selfDes(){},//自爆

    //装备属性
    atk : 0,//攻击力
    def : 0,//防御力
    defProp : 0,//百分比减伤
    atkNum : 1,//攻击次数
    atkSpd : 0,//攻击速度加成
    getAtkNum : function(){
        if(this.isArrow && data.race.name.startsWith('精灵-游侠将军')){
            return elementCfgs[this.name].atkNum + 1;
        }
        return elementCfgs[this.name].atkNum;
    },
    addWeight : 0,//额外增加的负重
    suck : 0,//吸血量，实际应该是0.xx这种百分比
    noWear : false,//是否永不磨损
    atkNeedItem : [],//攻击需要消耗的物品，为空则不消耗任何物品
    fixNeedDust : 1,//修复1点耐久需要消耗几点虚空粉尘
    quality : 0,//装备品质，0-4分别为【普通】【优秀】【精良】【传说】【神话】
    level : 1,//装备级别，不能获取超过装备级别的词条
    isThrow : false,//投掷物品
    enhancements : null,//附魔属性，{atk:1,def:2}
    hole : [],//镶嵌孔
    atkTags : null,//针对不同标签的怪物增加的伤害倍率，{tag : '标签名', rate : 0.5}
    atkAfter : null, //攻击后触发的方法
    jp : false, //是否极品装备

    //克系属性
    draw : 0,//吸引值，指对古神的吸引力

    //获取攻击力
    getAtk : function(item){
        if(!window.data){
            return 0
        }
        let num = 0;
        let atk = this.atk;
        try{
            //地球人投掷品伤害翻倍
            if(item.isThrow && data.race.name.startsWith('地球人')){
                num = 1;
            }
            //标签伤害增幅
            if(data.battleMonster && data.battleMonster.tags && item.atkTags && item.atkTags.length > 0){
                item.atkTags.forEach(e => {
                    if(data.battleMonster.tags.includes(e.tag)){
                        num += e.rate;
                    }
                })
            }
        }catch (e) {
            console.error(e)
        }
        if(this.name && elementCfgs[this.name].myAtk){
            atk = elementCfgs[this.name].myAtk(item);
        }

        if(num < -1){
            num = -1;
        }

        return parseInt(atk * (1 + num));
    },
    //获取HP
    getHp : function(){
        if(this.name && elementCfgs[this.name].myHp){
            return this.myHp ? this.myHp() : elementCfgs[this.name].myHp();
        }
        return this.hp;
    },

    //创建一个元素实例
    new : function(props){
        const element = new Element({id : getNewId(),...this,...props});
        if(element.hole > 0){
            element.hole = new Array(element.hole)
        }
        return element
    }
}
